/***************************************************************************
 *   Copyright (C) 2005 by Johan Maes   *
 *   on4qz@telenet.be   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/
#include "modegbr2.h"


modeGBR2::modeGBR2(esstvMode m,unsigned int len, bool tx):modeBase(m,len,tx)
{
}


modeGBR2::~modeGBR2()
{
}

void modeGBR2::setupParams(double clock)
{
	visibleLineLength=(lineLength(mode,clock)-fp-bp-2*blank-syncDuration)/3.;
}

modeBase::embState modeGBR2::rxSetupLine()
{
	// the start of the Scottie mode is always at the start of the green line 
  start=lineTimeTableRX[lineCounter];
 // if(subLine==0) addToLog(QString("modeGBR2: subLine %1, line=%2").arg(subLine).arg(lineCounter),DBMODES);
  switch(subLine)
		{

			case 0:
				calcPixelPositionTable(GREENLINE,FALSE);
				debugState=stColorLine0;
				pixelArrayPtr=greenArrayPtr;
				return MBPIXELS;
			case 1:
				debugState=stG1;
				marker=(unsigned int)round(start+blank+visibleLineLength);
				return MBRXWAIT;
			case 2:
				calcPixelPositionTable(BLUELINE,FALSE);
				debugState=stColorLine1;
				pixelArrayPtr=blueArrayPtr;
				return MBPIXELS;
			case 3:
				debugState=stFP;
				marker=(unsigned int)round(start+blank+fp+2*visibleLineLength);
				return MBRXWAIT;
			case 4:
				debugState=stSync;
				syncPosition=syncEndPosition;
				return MBSYNC;
			case 5:
				debugState=stBP;
				marker=(unsigned int)round(syncEndPosition+bp);
				return MBRXWAIT;
			case 6:
				calcPixelPositionTable(REDLINE,FALSE);
				debugState=stColorLine2;
				pixelArrayPtr=redArrayPtr;
				return MBPIXELS;
			case 7:
				debugState=stG1;
        marker=(unsigned int)round(lineTimeTableRX[lineCounter]);
				return MBRXWAIT;
			break;
			default:
				return MBENDOFLINE;
		}
}

void modeGBR2::calcPixelPositionTable(unsigned int colorLine,bool tx)
{
	unsigned int i;
	int ofx=0;
	if(tx) ofx=1;
	debugState=colorLine;
	switch (colorLine)
		{
			case GREENLINE:
				start+=bp;
       // addToLog(QString("gbr2: greenstart=%1").arg(start),DBMODES);

			break;
			case BLUELINE:
				start+=(bp+blank+visibleLineLength);
				syncEndPosition=(unsigned int)(start+visibleLineLength+fp+syncDuration);

       // addToLog(QString("gbr2: bluestart=%1").arg(start),DBMODES);

			break;
			case REDLINE:
				start+=bp+fp+syncDuration+bp+blank+2.*visibleLineLength;
       // addToLog(QString("gbr2: redstart=%1").arg(start),DBMODES);

			break;
		}
  for(i=0;i<activeSSTVParam->numberOfPixels;i++)
		{
      pixelPositionTable[i]=(unsigned int)round(start+(((float)(i+ofx)*visibleLineLength)/activeSSTVParam->numberOfPixels));
		}
}


unsigned long modeGBR2::adjustSyncPosition(unsigned long syncPos)
{
  if(syncPos<(unsigned long)(fp+2*visibleLineLength+2*blank+syncDuration))
  {
    return syncPos+(unsigned long)(bp+visibleLineLength);
  }
  else
  {
    return syncPos-(unsigned long)(fp+2*visibleLineLength+2*blank+syncDuration);
  }
}

modeBase::embState modeGBR2::txSetupLine()
{
  start=lineTimeTableTX[lineCounter];
 // if(subLine==0) addToLog(QString("modeGBR2: subLine %1, line=%2").arg(subLine).arg(lineCounter),DBMODES);
  switch(subLine)
		{
			case 0:
				calcPixelPositionTable(GREENLINE,TRUE);
				pixelArrayPtr=greenArrayPtr;
				return MBPIXELS;
			case 1:
				txFreq=1500.;
				txDur=(unsigned int)rint(blank);
				return MBTXGAP;
			case 2:
				calcPixelPositionTable(BLUELINE,TRUE);
				pixelArrayPtr=blueArrayPtr;
				return MBPIXELS;
			case 3:
				txFreq=1500;
				txDur=(unsigned int)rint(fp);
				return MBTXGAP;
			case 4:
				txFreq=1200;
				txDur=(unsigned int)rint(syncDuration);
				return MBTXGAP;
			case 5:
				txFreq=1500;
				txDur=(unsigned int)rint(bp);
				return MBTXGAP;	
			case 6:
				calcPixelPositionTable(REDLINE,TRUE);
				pixelArrayPtr=redArrayPtr;
				return MBPIXELS;
			case 7:
				txFreq=1500.;
				txDur=(unsigned int)rint(blank);
				return MBTXGAP;
			default:
				return MBENDOFLINE;
		}
}
